// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'dart:async';

import 'package:ducene/analysis.dart';
import 'package:ducene/document.dart';
import 'package:ducene/index.dart';
import 'package:ducene/search.dart';
import 'package:ducene/store.dart';

class IndexWriterBenchmark {

  static Future main() async {
    for (int i = 0; i < 10; i++) {
      IndexWriterBenchmark bench = new IndexWriterBenchmark();
      await bench.run();
    }
  }

  Future run() async {
    Analyzer ws = new WhitespaceAnalyzer(); // slow new NGramAnalyzer(2);
    IndexDirectory directory = new RAMIndexDirectory();
    IndexWriter writer = new IndexWriter(directory);
    int iteration = 10000;
    Iterable<Document> docs = new Iterable.generate(iteration, (i) {
      return new Document()
          .append('id', i.toString())
          .append('text', text, analyzer: ws);
    });
    Stopwatch stopwatch = new Stopwatch();
    stopwatch.start();
    await writer.write(docs);
    print('indexed ${stopwatch.elapsed.inMilliseconds} ms');
  }
}

const String text = '''
Dart is a general-purpose programming language originally developed
by Google and later approved as a standard by Ecma (ECMA-408).[4]
It is used to build web, server and mobile applications,
and for Internet of Things (IoT) devices.[5]
It is open-source software under a BSD license.
''';

class IndexSearcherBenchmark {

  static Future main() async {
    for (int i = 0; i < 10; i++) {
      IndexSearcherBenchmark bench = new IndexSearcherBenchmark();
      await bench.setup();
      await bench.run();
    }
  }

  Analyzer _st;
  IndexDirectory _directory;

  Future setup() async {
    _st = new StandardAnalyzer();
    _directory = new RAMIndexDirectory();
    IndexWriter writer = new IndexWriter(_directory);
    int iteration = 10000;
    Iterable<Document> docs = new Iterable.generate(iteration, (i) {
      return new Document()
          .append('id', i.toString())
          .append('text', text, analyzer: _st);
    });
    //Stopwatch stopwatch = new Stopwatch();
    //stopwatch.start();
    await writer.write(docs);
    //print('indexed ${stopwatch.elapsed.inMilliseconds} ms');
  }

  Future run() async {
    IndexReader reader = await DirectoryReader.open(_directory);
    IndexSearcher searcher = new IndexSearcher(reader);
    BoolQuery query = new BoolQuery(Op.or).append('text', 'software', analyzer: _st);
    Stopwatch stopwatch = new Stopwatch();
    stopwatch.start();
    TopDocs hits = await searcher.search(query, 10);
    assert (10000 == hits.totalHits);
    print('searched ${stopwatch.elapsed.inMilliseconds} ms');
  }
}

Future main() async {
  await IndexWriterBenchmark.main();
  await IndexSearcherBenchmark.main();
}