// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import '../../src/analysis/analyzer.dart';

class Document {
  List<Field> fields = [];
  Document add(Field field) {
    fields.add(field);
    return this;
  }
  static Analyzer defaultAnalyzer = new KeywordAnalyzer();
  Document append(String name, Object text, {Analyzer analyzer, bool stored: true}) {
    List<String> txt = [];
    if (text is String) {
      txt.add(text);
    } else if (text is Iterable) {
      txt = (text.toList() as List<String>);
    }
    add(new Field(name, txt, (analyzer == null)? defaultAnalyzer : analyzer, stored));
    return this;
  }
  String get(String name) {
    var values = getValues(name);
    return (values != null) ? values[0] : null;
  }
  List<String> getValues(String name) {
    List<Field> fs = fields.where((e) => e.name == name).toList();
    Field f = (fs.length != 0) ? fs[0] : new Field(null,null,null,null);
    return (f.text != null) ? f.text : null;
  }
}
class Field {
  final String name;
  final List<String> text;
  final Analyzer analyzer;
  final bool stored;
  Field(this.name, this.text, this.analyzer, this.stored);
}