// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'dart:async';
import 'dart:collection';

import '../../../src/index/index_reader.dart';

class FieldFacet {
  static Future<Map<String,FacetValue>> getCount(Set<int> docSet, String field, IndexReader reader) async {
    Map<String,Set<int>> setAcc = {};
    Map<String,double> scoreAcc = {};
    var maxDoc = await reader.maxDoc();
    int i = 0;
    for (String term in await reader.terms(field)) {
      Set<int> unionSet = (await reader.postings(field, term)).toSet();
      Set<int> resultSet = unionSet.intersection(docSet);
      if (resultSet.length > 0) {
        setAcc[term] = resultSet;
        double score = resultSet.length + (maxDoc/(i+1) * 0.00000001);
        scoreAcc[term] = score;
      }
      i++;
    }
    SplayTreeMap<double,String> sort
      = new SplayTreeMap.fromIterables(scoreAcc.values, scoreAcc.keys, (a, b) {
      return b.compareTo(a);
    });
    Map<String,FacetValue> ret = {};
    for (var term in sort.values) {
      ret[term] = new FacetValue(setAcc[term]);
    }
    return ret;
  }
}
class FacetValue {
  int value;
  Set<int> docSet;
  FacetValue(this.docSet) {
    value = docSet.length;
  }
}