// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'dart:async';

import 'package:firebase3/firebase.dart' as fb;

import '../../../src/index/index_reader.dart';
import '../../../src/store/index_directory.dart';

class FirebaseIndexDirectory implements IndexDirectory {
  FirebaseIndexStreams _indexStreams;
  FirebaseDataStreams _dataStreams;
  FirebaseIndexDirectory(this._indexStreams, this._dataStreams);

  @override Future initOutput() { return new Future.value(Null); }

  @override Future outputLiveDocs(Set<int> docIds) {
    throw new Exception('UnsupportedOperationException');
  }

  @override Future outputIndex(String name, Map<String,DocSet> index) {
    throw new Exception('UnsupportedOperationException');
  }

  @override Future outputData(String name, Map<int,DocData> data) {
    throw new Exception('UnsupportedOperationException');
  }

  @override void initInput() { }

  @override LiveDocsSource inputLiveDocs() {
    return new LiveDocsSource(new FirebaseLiveDocsStreams());
  }

  @override IndexSource inputIndex() {
    return new IndexSource(_indexStreams);
  }

  @override DataSource inputData() {
    return new DataSource(_dataStreams);
  }
}
class FirebaseLiveDocsStreams extends LiveDocsStreams {
  FirebaseLiveDocsStreams();
  @override Future readFully(DocSet to) async {

  }
}
class FirebaseIndexStreams extends IndexStreams {
  bool _readFully;
  bool loaded = false;

  @override Future readFully(Map to) async {
    _readFully = true;
    await FirebaseUtils._getFields("idx").then((fields) {
      for (String field in fields) {
        _getInvertedIndex("idx/"+field).then((invertedIndex) => to[field] = invertedIndex);
      }
    }).then((_) {loaded = true;});
  }
  Future<Map<String,DocSet>> _getInvertedIndex(String path) async {
    var invertedIndex = <String,DocSet>{};
    var query = fb.database().ref(path).orderByKey();
    var event = await query.once("value");
    event.snapshot.forEach((e) {
      String key = e.val()["term"];
      Set<int> values = (e.val()["postings"] as String).split(',').map((s) => int.parse(s)).toSet();
      invertedIndex[key] = new DocSet(values);
    });
    return invertedIndex;
  }

  @override Future readTerms(String field, Map<String,Map<String,DocSet>> to) async {
    if (!_readFully) await readFully(to);
  }

  @override Future readPostings(String field, String term, Map<String,Map<String,DocSet>> to) async {
    if (!_readFully) await readFully(to);
  }
}
class FirebaseDataStreams extends DataStreams {
  bool _readFully;
  bool loaded = false;

  @override Future readFully(Map to) async {
    _readFully = true;
    await FirebaseUtils._getFields("data").then((fields) {
      for (String field in fields) {
        _getStoredData("data/"+field).then((storedData) => to[field] = storedData);
      }
    }).then((_) {loaded = true;});
  }
  Future<Map<int,DocData>> _getStoredData(String path) async {
    var storedData = <int,DocData>{};
    var query = fb.database().ref(path).orderByKey();
    var event = await query.once("value");
    event.snapshot.forEach((e) {
      int key = int.parse(e.key);
      List<String> values = (e.val() as String).split('\$');
      storedData[key] = new DocData(values);
    });
    return storedData;
  }

  @override Future readFields(Map<String,Map<int,DocData>> to) async {
    if (!_readFully) await readFully(to);
  }

  @override Future readValues(String field, int docId, Map<String,Map<int,DocData>> to) async {
    if (!_readFully) await readFully(to);
  }
}
class FirebaseUtils {
  static Future<List<String>> _getFields(String extension) async {
    List<String> fields = [];
    var query = fb.database().ref(extension).orderByKey();
    var event = await query.once("value");
    event.snapshot.forEach((e) => fields.add(e.key));
    return fields;
  }
}