// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'package:ducene/document.dart';
import 'package:ducene/index.dart';
import 'package:ducene/search.dart';
import 'package:ducene/store.dart';
import 'package:test/test.dart';

void main() {
  group('A group of facet', () {

    test('test correct facet counts', () async {
      IndexDirectory indexDirectory = new RAMIndexDirectory();
      IndexWriter writer = new IndexWriter(indexDirectory);

      List<Document> d1 = [];
      for (int i = 0; i < 10; i++) {
        Document doc = new Document();
        doc.append('id', i.toString());
        doc.append('text', 'text');
        doc.append('cat', 'A');
        d1.add(doc);
      }
      List<Document> d2 = [];
      for (int i = 10; i < 15; i++) {
        Document doc = new Document();
        doc.append('id', i.toString());
        doc.append('text', 'text');
        doc.append('cat', ['B','C']);
        d2.add(doc);
      }
      var docs = <Document>[];
      docs.addAll(d1);
      docs.addAll(d2);

      await writer.write(docs);

      IndexReader reader = await DirectoryReader.open(indexDirectory);
      IndexSearcher searcher = new IndexSearcher(reader);

      BoolQuery q = new BoolQuery();
      q.append('text', 'text');
      TopDocs hits = await searcher.search(q, 100);

      expect(hits.totalHits, 15);

      //Stopwatch stopwatch = new Stopwatch();
      //stopwatch.start();
      //for (int i = 0; i < 100000; i++) {
        var catFacet = await FieldFacet.getCount(hits.docSet,'cat',reader);
        expect(catFacet['A'].value, 10);
        expect(catFacet['B'].value, 5);
        expect(catFacet['C'].value, 5);
      //}
      //print(stopwatch.elapsedMilliseconds);
    });

  });
}

