// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'dart:io';

import 'package:ducene/document.dart';
import 'package:ducene/index.dart';
import 'package:ducene/search.dart';
import 'package:ducene/store.dart';
import 'package:ducene/store_sembast.dart';
import 'package:test/test.dart';

void main() {

  group('A group of directory', () {
    SembastIndexDirectory directory;
    Directory dir = new Directory('test-dir');

    setUp(() {
      if (dir.existsSync()) dir.deleteSync(recursive: true);
      directory = new SembastIndexDirectory("test-dir", ["id","text"], ["id","text"]);
    });

    tearDown(() {
      if (dir.existsSync()) dir.deleteSync(recursive: true);
    });

    test('test 001', () async {
      IndexWriter writer = new IndexWriter(directory);
      List<Document> docs = [];
      for (int i = 0; i < 10; i++) {
        Document doc = new Document();
        doc.append("id", i.toString());
        doc.append("text", "x");
        docs.add(doc);
      }
      await writer.write(docs);

      IndexReader reader = await DirectoryReader.open(directory);
      IndexSearcher searcher = new IndexSearcher(reader);

      expect(await searcher.count(new BoolQuery().append('id', '5')), 1);
      expect(await searcher.count(new BoolQuery().append('text', 'x')), 10);

    });

  });
}

